precision highp float;
uniform sampler2D inputTexture;
varying vec2 textureCoordinate;

uniform vec2 key_position;
uniform vec3 key_color;

uniform int mode; // 0 - sample key color from key_position, otherwise use uniform key_color

uniform float min_threshold;
uniform float max_threshold;

vec2 RGB_to_AB(vec3 c) {
	float labA, labB;

    // standard illuminant
    // see: https://en.wikipedia.org/wiki/Illuminant_D65
    vec3 D65 = vec3(0.9505, 1.0, 1.089);

    vec3 cExp = pow((c + 0.055)/1.055, vec3(2.2));
    vec3 cLin = c / 12.92;
    
    // apply linear and exponential transform to different ranges
    vec3 cMix = mix(cLin, cExp, vec3(greaterThan(c, vec3(0.04045))));
                  
    // change basis
    vec3 f = mat3(0.4124, 0.2126, 0.0193,
                  0.3576, 0.7152, 0.1192,
                  0.1805, 0.0722, 0.9505) * cMix;
	
    // scale based on standard illuminant
    f = clamp(f, vec3(0), D65) / D65;
    
    vec3 fExp = pow(f, vec3(1.0/3.0));
    vec3 fLin = 7.787*f + 16.0/116.0;

    // apply linear and exponential transform to different ranges
    vec3 fMix = mix(fLin, fExp, vec3(greaterThan(f, vec3(0.008856))));
    
    // finally compute A and B
	labA = 500.0 * (fMix.x - fMix.y); // A range: [-86.185,  98.254]
	labB = 200.0 * (fMix.y - fMix.z); // B range: [-107.863, 94.482]
    
    // normalize B to [0, 1], A should weigh less in Euclidean distance metric
	return vec2((labA+86.185) / 202.345, (labB+107.863) / 202.345);
}

float desaturate(vec3 p) {
    return dot(p, vec3(0.299, 0.587, 0.114));
} 

float cubestep(float a, float b, float x) {
    return pow(clamp((x-a) / (b-a), 0.0, 1.0), 3.);
}

void main() {
    vec4 texColor = texture2D(inputTexture, textureCoordinate);

    // sample key color from image or take the value from uniform
    vec3 key = mode == 0 ? texture2D(inputTexture, key_position).rgb : key_color;
    
    // convert colors to LAB space
    vec2 texAB = RGB_to_AB(texColor.rgb);
    vec2 keyAB = RGB_to_AB(key);
    
    // calculate squared distance
    float keyDistSqr = dot(keyAB - texAB, keyAB - texAB);

    // calculate alpha by applying step function to squared distance
    texColor.a = cubestep(min_threshold, max_threshold, keyDistSqr);
    
    // convert color to grayscale
    float desat = desaturate(texColor.rgb);

    // desaturate the image color based on alpha
    texColor.rgb = mix(vec3(desat), texColor.rgb, texColor.a);

    gl_FragColor = texColor;
}